(function () {
    'use strict';

    angular.module('soter')
        .constant('AUTH_EVENTS', {
            loginSuccess: 'auth-login-success',
            loginFailed: 'auth-login-failed',
            logoutSuccess: 'auth-logout-success',
            sessionTimeout: 'auth-session-timeout',
            notAuthenticated: 'auth-not-authenticated',
            notAuthorized: 'auth-not-authorized'
        })
        .factory('AuthService', ['$http', '$q', 'Session', AuthService]);
    function AuthService($http, $q, Session) {
        var authService = {};

        authService.login = function (credentials) {
            return $http
                .post('/v1/login', credentials)
                .success(function (data) {
                    Session.create(data);
                    return data;
                });
        };

        authService.retrieve = function () {
            return $http
                .get('/v1/profile')
                .success(function (data) {
                    Session.create(data);
                    return data;
                });
        };

        authService.logout = function () {
            return $http
                .post('/logout', {})
                .success(function (res) {
                    Session.destroy();
                    return res;
                });
        }

        authService.isAuthenticated = function () {
            return Session.isAuthenticated;
        }
        authService.profile = function () {
            if (!Session.hasSession) {
                return authService.retrieve().then(function (data) {
                    return {
                        id: Session.id,
                        userId: Session.userId,
                        name: Session.name,
                        permissions: Session.permissions,
                        isAuthenticated: Session.isAuthenticated
                    }
                });
            }
            else {
                var deferred = $q.defer();
                deferred.resolve({
                    id: Session.id,
                    userId: Session.userId,
                    name: Session.name,
                    permissions: Session.permissions,
                    isAuthenticated: Session.isAuthenticated
                });
                return deferred.promise;
            }
        }

        authService.isAuthorized = function (permission) {
            return (authService.isAuthenticated() && Session.permissions.indexOf(permission) !== -1);
        };

        return authService;
    }

})();
